> Our goal with this project is to have a super simple + low barrier to entry for development. All you should need is nodejs + git. There should be be no complexity in the development setup or workflow 🌹

# Requirements
You need:

* git (If you are windows please install the version of git that has git bash and run `npm install` from that).
* node v5+

# Setup
```
git clone https://github.com/alm-tools/alm.git
cd alm
```

We highly recommend you use [`nodemon`](https://github.com/remy/nodemon), just run it from the project root:

```
npm install nodemon -g
nodemon
```

Alternatively if you want to test `alm` on another folder e.g. `your project` (i.e not our folder with our test files) you can make the dev version of `alm` avialable globally with:

```
npm install
npm link
```

# Workflow

Once you have `nodemon` running, if you edit any front-end ts file our Webpack setup will reload the front end only. Make a change to some backend file and the app restarts :rose:

# IDE

You are free to use any IDE but you should also run `npm run tscw` to run the same TypeScript version as alm in watch mode.

# Webpack
All our front-end js gets minified into `bundle.js`. This is what gets npm deployed.

For dev time we have a setup (all code in `devtime.ts`) where we start a `WebpackDevServer` and proxy the `bundle.js` requests to it, additionally loading the hot loader to refresh our UI if any front-end JS changes.

The meanings of some key folders:
```
src/app            -> all our front-end (main entry point called `main.ts`)
src/public/build   -> bundle.js and other generated resources go here
src/server         -> our backend code (main entry point called `server.ts`)
src/common         -> Shared between server /client. No server or client only code.
webpack.config.ts  -> our webpack config shared between devtime and what gets npm deployed
```

In addition to that we make it easier to test the app as it would appear after npm publish:

* Visit `/prod` to see the app as it would work after npm publish and then test app. Live reload will be disabled.
* Visit `/dev` to go back to the dev time workflow.

More reading (You don't need to read these ... but if you are bored):
* [Hot reloading docs](https://github.com/webpack/docs/wiki/hot-module-replacement-with-webpack)
* [Some ideas borrowed from here](http://www.christianalfoni.com/articles/2015_04_19_The-ultimate-webpack-setup)

# Server Folder Structure

* `server/workers` we run various background workers. They are present here.
* `server/lang` our main API stuff on top of the core TypeScript Language Service.

# How does the frontend talk to the backend
Documented in the [principles of ASYNC][async].

# How is global state managed
State for visual components in mostly managed by redux which we have made type-safe. [Checkout the docs][redux].

# Conventions
Please see [conventions.md][conventions]

# Deployment
Please see [deployment.md][deployment]

# Various NPM commands

### After moving a new machine
I quite often work on my personal laptop during my 1 hour one way commute and need to setup the machine quickly, so on the new machine:

```
npm run resume
```

This will just run the right things again (pull + install + initial compile).

### Updating TypeScript Version
We use [NTypeScript](https://github.com/TypeStrong/ntypescript). NTypeScript keeps itself updated with Microsoft/Typescript every night. To update the version used by alm.tools simply run
```
npm run uts
```
This will install the latest version of NTypeScript and run a build to make sure everything still compiles.

> Note: NTypeScript makes the `ts` variable available globally in our backend / frontend  code bases. So you don't need to do any explicit `import * as ts` in new files.

> Also be sure to checkout the [deployment guide][deployment].

> We have a similar target `npm run ucm` that updates CodeMirror.

### Running tsc in watch mode
If you did something which resulted in a lot of errors you can run tsc in watch mode in a new window:

```
npm run tscw
```

### Working on another branch
Get the latest git commits from that branch. Next all you need to do is `npm install`. Now run `nodemon` like you would normally do.

# Idea Tracking
Since I am still doing a lot of *thinking* then *rethinking* I'm trying not to put too much stuff in the [issues][issues] as it spams the watchers. Instead I've been using a private trello but its very *raw* (like a pen an paper). Feel free to ask me questions on [gitter][gitter] about what's in my headspace. I try to be open about it on [twitter][twitter] anyways.

[issues]: https://github.com/alm-tools/alm/issues
[gitter]: https://gitter.im/alm-tools/alm
[twitter]: https://twitter.com/basarat
[async]: ./async.md
[conventions]: ./conventions.md
[deployment]: ./deployment.md
[redux]: ./redux.md
